// DTree.cpp: implementation of the DTree class.
//
//////////////////////////////////////////////////////////////////////

#include "DTree.h"
#include "DTNode.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DTree::DTree(int patterns, int attributes, float **elements)
{
	no_of_attributes = attributes;
	no_of_patterns = patterns;
	root = new DTNode(no_of_patterns, no_of_attributes, elements,-1, 0);
	max_level = 0;
	no_of_nodes = 1;
}

DTree::~DTree()
{

}

DTNode* DTree::impure(DTNode* p, double& p_0_given_t, double& p_1_given_t)
{
	//if p is impure, return p
	//else if left  child of p is impure, return l_child
	//else if right child of p is impure, return r_child
	//else return null
	double i_t;
	int count_0=0;
	int count_1=0;
	DTNode* result = 0;
	if(p->is_leaf()){
		for(int i=0; i<p->get_no_of_patterns(); i++){
			if( (p->get_elements())[i][this->get_no_of_attributes()-1] == 0)
				count_0++;
			else
				count_1++;
		}
		
		if(count_0 !=0 && count_1 != 0){
			p_0_given_t = (double)count_0 / p->get_no_of_patterns();
			p_1_given_t = (double)count_1 / p->get_no_of_patterns();

			i_t = p_0_given_t*log(p_0_given_t)/log(2) + p_1_given_t*log(p_1_given_t)/log(2);
			i_t = i_t * -1;
			return p;
		}

	}
	result = 0;
	if(p->l_child())
		result = this->impure(p->l_child(),p_0_given_t,p_1_given_t);
	if(!result && p->r_child()){
		result = this->impure(p->r_child(),p_0_given_t,p_1_given_t);
	}
	
	return result;
}

void DTree::traverse(DTNode* p, DTNode* arr,int& index){
	int current;
	current = index;
	if(p->is_leaf())
		arr[current].set_decision(p->get_decision());
	else{
		arr[current].set_decision_attribute(p->get_decision_attribute());
		arr[current].set_decision_point(p->get_decision_point());
		arr[current].set_l_c_index(++index);
		traverse(p->l_child(),arr,index);
		arr[current].set_r_c_index(++index);
		traverse(p->r_child(),arr,index);
	}	
}

void DTree::build(DTNode* arr, DTNode* p, int index){
	p->set_decision_attribute(arr[index].get_decision_attribute());
	p->set_decision(arr[index].get_decision());
	p->set_decision_point(arr[index].get_decision_point());
	if(p->get_decision() == -1){
		p->set_l_child(&arr[arr[index].get_l_c_index()]);
		p->set_r_child(&arr[arr[index].get_r_c_index()]);
		build(arr, p->l_child(), arr[index].get_l_c_index());
		build(arr, p->r_child(), arr[index].get_r_c_index());
	}
}

int DTree::follow(double* prop, int size){
	DTNode* p = root;
	int i;
	while(p->get_decision()==-1){
		i = p->get_decision_attribute();
		if(prop[i] <= p->get_decision_point()){
			p = p->l_child();
		}else{
			p = p->r_child();
		}
	}
	return p->get_decision();
}
